@@ -1,12 +1,14 @@
 #!/usr/bin/env python
-# (c) Copyright 2009 Cloudera, Inc.
+# (c) Copyright 2010 Cloudera, Inc.
 import logging
 import os
 import re
 import simplejson
+import sys
 
 from git_command import GitCommand
 from git_repo import GitRepo
+import trace
 
 
 class Manifest(object):
@@ -199,6 +201,19 @@ class Project(object):
     self.tracker = tracker
     self.from_remote = from_remote
     self.remote_project_name = remote_project_name or name
+    # If this project tracks hash or indirect, it may already be up-to-date
+    try:
+      rev = self.git_repo.rev_parse("HEAD")
+      remote_ref = self.tracker.remote_ref
+      trace.Trace("<%s> %s (local) -- %s (remote)" %
+                  (name, rev, remote_ref))
+      self.__is_uptodate = remote_ref == rev
+    except Exception, ex:                # Error due to uninitialized project
+      trace.Trace("Non fatal error %s", ex)
+      self.__is_uptodate = False
+
+  def __str__(self):
+    return "Project %s" % (self.name,)
 
   @staticmethod
   def from_dict(manifest, name, data):
@@ -272,6 +287,12 @@ class Project(object):
   def git_repo(self):
     return GitRepo(self.dir)
 
+  def is_uptodate(self):
+    return self.__is_uptodate
+
+  def set_uptodate(self):
+    self.__is_uptodate = True
+
   def is_cloned(self):
     return self.git_repo.is_cloned()
 
